@@ -7,6 +7,8 @@ import ijson
 import json
 import csv
 import inspect
+import urllib
+from datetime import datetime, timedelta
 
 class Conn:
     """Setup the connexion to Eulerian Technologies API.
@@ -40,6 +42,7 @@ class Conn:
         self.__print_log = print_log
         self.__http_headers = { "Authorization" : f"Bearer {api_key}" }
         self.__base_url = f"https://{gridpool_name}.api.eulerian.{datacenter}"
+        self.__api_key = api_key
 
         overview_url = f"{self.__base_url}/ea/v2/er/account/authtree.json"
         overview_json = requests.get(
@@ -48,11 +51,7 @@ class Conn:
         ).json()
 
         if self.__has_api_error(overview_json):
-            raise Exception(f"Connexion unsuccessful for:\
-                \n gridpool_name={gridpool_name}\
-                \n datacenter={datacenter}\
-                \n api_key={api_key}\
-            ")
+            raise SystemError(f"Error for url={overview_url}")
         else:
             self.__log("Connexion ok")
 
@@ -107,8 +106,8 @@ class Conn:
             payload = {},
             status_waiting_seconds = 5,
             output_directory = None,
-            output_filename = None,
             override_file = False,
+            n_days_slice = 31,
     ):
 
         """ Fetch datamining data from the API into a gzip compressed file
@@ -121,9 +120,6 @@ class Conn:
             datamining_type : str, obligatory
                 The targeted datamining (isenginerequest, actionlogorder, scart ,estimate, order)
 
-            jobrun_id : str, optional
-                The jobrun_id to download directly from a previously requested jobrun
-
             payload : dict, optional
                 The datamining payload that contains the requested data
 
@@ -133,37 +129,48 @@ class Conn:
             output_directory : str, optional
                 The local targeted  directory
 
-            output_filename: str, optional
-                If not set, the file will be created in the current working directory with a default name
-
             override_file : bool, optional
                 If set to True, will override output_path2file (if exists) with the new datamining content
                 Default: False
 
+            n_days_slice: int, optional
+                Split datamining query into days slice to reduce server load
+                Default: 31
+
         Returns
         -------
-        str
-            The output_path2file containing the downloaded datamining data
+        list
+            A list of path2file
         """
 
-        if not output_directory:
-            output_directory = ""
+        def download (
+            website_name: str,
+            datamining_type: str,
+            payload: dict,
+            date_format : str,
+            dt_date_from: timedelta(),
+            dt_date_to: timedelta(),
+            view_id: int,
+            status_waiting_seconds: int,
+        ):
+            date_from = dt_date_from.strftime(date_format)
+            date_to = dt_date_to.strftime(date_format)
+            date_from_file = date_from.replace("/", "_")
+            date_to_file = date_to.replace("/", "_")
 
-        if self.__create_output_directory(output_directory): # error
-            return 0
+            output_filename = f"{website_name}_{datamining_type}_view_{view_id}_from_{date_from_file}_to_{date_to_file}.csv.gz"
+            output_path2file = os.path.join(output_directory, output_filename)
 
-        if not output_filename:
-            date_from = payload["date-from"].replace("/", "_")
-            date_to = payload["date-from"].replace('/', "_")
-            output_filename = f"{website_name}_{datamining_type}_{date_from}_{date_to}.csv.gz"
+            if self.__skipping_download(output_path2file, override_file):
+                return { "path2file" : output_path2file }
 
-        output_path2file = os.path.join(output_directory, output_filename)
+            payload['date-from'] = date_from
+            payload['date-to'] = date_to
 
-        if self.__skipping_download(output_path2file, override_file):
-            return output_path2file
-            
-        if not jobrun_id:
             search_url = f"{self.__base_url}/ea/v2/ea/{website_name}/report/{datamining_type}/search.json"
+            search_url_debug = f"{self.__base_url}/ea/v2/{self.__api_key}/ea/{website_name}/report/{datamining_type}/search.json"
+            self.__log(f"http get url {search_url_debug}?{urllib.parse.urlencode(payload)}")
+
             search_json = requests.get(
                 url=search_url,
                 params=payload,
@@ -171,79 +178,151 @@ class Conn:
             ).json()
 
             if self.__has_api_error(search_json):
-                return 0
+                raise SystemError(f"Error for url={search_url}?{urllib.parse.urlencode(payload)}")
 
             jobrun_id = search_json["jobrun_id"]
 
-        status_url =  f"{self.__base_url}/ea/v2/ea/{website_name}/report/{datamining_type}/status.json"
-        status_payload = { "jobrun-id" : jobrun_id }
-        ready = False
+            status_url =  f"{self.__base_url}/ea/v2/ea/{website_name}/report/{datamining_type}/status.json"
+            status_payload = { "jobrun-id" : jobrun_id }
+            ready = False
 
-        if status_waiting_seconds < 5 or not isinstance(status_waiting_seconds, int):
-            status_waiting_seconds = 5
+            if status_waiting_seconds < 5 or not isinstance(status_waiting_seconds, int):
+                status_waiting_seconds = 5
 
-        while not ready:
-            self.__log(f'Waiting for jobrun_id={jobrun_id} to complete')
-            time.sleep(status_waiting_seconds)
-            status_json = requests.get(
-                url=status_url,
-                params=status_payload,
-                headers=self.__http_headers
-            ).json()
+            while not ready:
+                self.__log(f'Waiting for jobrun_id={jobrun_id} to complete')
+                time.sleep(status_waiting_seconds)
+                status_json = requests.get(
+                    url=status_url,
+                    params=status_payload,
+                    headers=self.__http_headers
+                ).json()
 
-            if self.__has_api_error(status_json):
+                if self.__has_api_error(status_json):
+                    raise SystemError(f"Error for url={status_url}?{urllib.parse.urlencode(status_payload)}")
+
+                if status_json["jobrun_status"] == "COMPLETED":
+                    ready = True
+
+            self.__log('Downloading data')
+            download_url = f"{self.__base_url}/ea/v2/ea/{website_name}/report/{datamining_type}/download.json"
+            download_payload = { 'output-as-csv' : 0, 'jobrun-id' : jobrun_id }
+            output_path2file_temp = f"{output_path2file}.tmp"
+
+            with  open(output_path2file_temp, 'wb') as f:
+                r = requests.get(
+                        download_url,
+                        params = download_payload,
+                        headers = self.__http_headers,
+                        stream=True
+                )
+                try:
+                    self.__log(f"Content-Length is {int(r.headers['Content-Length'])/(1024*1024)} MBs")
+
+                except Exception as e:
+                    self.__log("Could not read Content-Length header")
+
+                for chunk in r.iter_content(1024 * 1024 * 5): # 5MB
+                    f.write(chunk)
+
+            self.__log(f"JSON data downloaded into path2file={output_path2file_temp}")
+            self.__log("Converting JSON to CSV...")
+
+            with gzip.open(output_path2file, "wt") as csvfile:
+                csvwriter = csv.writer(csvfile, delimiter=';')
+
+                with open(output_path2file_temp) as f:
+                    headers_object = ijson.items(f, "data.fields")
+                    for headers in headers_object:
+                        csv_headers = [header["header"] for header in headers]
+                        csvwriter.writerow(csv_headers)
+                  
+                with open(output_path2file_temp) as f:
+                    rows_object = ijson.items(f, "data.rows")
+                    for rows in rows_object:
+                        for row in rows:
+                            csvwriter.writerow(row)
+
+            # removing temp file
+            if os.path.exists(output_path2file_temp):
+                self.__log(f"Deleting temp_file={output_path2file_temp}")
+                os.remove(output_path2file_temp)
+
+            self.__log(f"Output csv path2file={output_path2file}")
+            return { "path2file" : output_path2file }
+
+        l_path2file = [] # store each file for n_days_slice
+        l_allowed_datamining_types = ["order", "estimate", "isenginerequest", "actionlog", "scart"]
+        date_format = "%m/%d/%Y"
+
+        if datamining_type not in l_allowed_datamining_types:
+            raise ValueError(f"datamining_type={datamining_type} not allowed, use one of the following: {', '.join(l_allowed_datamining_types)}")
+
+        if not isinstance(n_days_slice, int):
+            raise TypeError(f"type of 'n_days_slice' expected: 'int' but got: '{type(n_days_slice)}'")
+
+        date_from = payload["date-from"] if "date-from" in payload else None
+        if not date_from:
+            raise ValueError("missing parameter=date-from in payload object")
+
+        date_to = payload['date-to'] if 'date-to' in payload else None
+        if not date_to:
+            raise ValueError("missing parameter=date-from in payload object")
+
+        n_days_slice = timedelta(days=n_days_slice)
+        dt_date_from = datetime.strptime(date_from, date_format)
+        dt_date_to = datetime.strptime(date_to, date_format)
+
+        if dt_date_from > dt_date_to:
+            raise ValueError("'date-from' cannot occur later than 'date-to'")
+
+        if not output_directory:
+            output_directory = ""
+
+        if self.__create_output_directory(output_directory): # error
+            return 0
+
+        view_id = int(payload["view-id"]) if "view-id" in payload else 0
+
+        while dt_date_from + n_days_slice <= dt_date_to:
+            dt_tmp_date_to = dt_date_from + n_days_slice
+            d_output_path2file = download(
+                website_name = website_name,
+                datamining_type = datamining_type,
+                payload = payload,
+                dt_date_from = dt_date_from,
+                dt_date_to = dt_tmp_date_to,
+                status_waiting_seconds = status_waiting_seconds,
+                view_id = view_id,
+                date_format = date_format,
+            )
+            if "error" in d_output_path2file and d_output_path2file["error"]:
                 return 0
 
-            if status_json["jobrun_status"] == "COMPLETED":
-                ready = True
+            l_path2file.append(d_output_path2file["path2file"])
+            dt_date_from += n_days_slice
 
-        self.__log('Downloading data')
-        download_url = f"{self.__base_url}/ea/v2/ea/{website_name}/report/{datamining_type}/download.json"
-        download_payload = { 'output-as-csv' : 0, 'jobrun-id' : jobrun_id }
-        output_path2file_temp = f"{output_path2file}.tmp"
+        else:
+            if dt_date_from == dt_date_to and len(l_path2file): 
+                pass
 
-        with  open(output_path2file_temp, 'wb') as f:
-            r = requests.get(
-                    download_url,
-                    params = download_payload,
-                    headers = self.__http_headers,
-                    stream=True
-            )
-            try:
-                self.__log(f"Content-Length is {int(r.headers['Content-Length'])/(1024*1024)} MBs")
+            else: # last slice
+                d_output_path2file = download(
+                    website_name = website_name,
+                    datamining_type = datamining_type,
+                    payload = payload,
+                    dt_date_from = dt_date_from,
+                    dt_date_to = dt_date_to,
+                    status_waiting_seconds = status_waiting_seconds,
+                    view_id = view_id,
+                    date_format = date_format,
+                )
+                if "error" in d_output_path2file and d_output_path2file["error"]:
+                    return 0
 
-            except Exception as e:
-                self.__log("Could not read Content-Length header")
-
-            for chunk in r.iter_content(1024 * 1024 * 5): # 5MB
-                f.write(chunk)
-
-        self.__log(f"JSON data downloaded into path2file={output_path2file_temp}")
-        self.__log("Converting JSON to CSV...")
-
-        with gzip.open(output_path2file, "wt") as csvfile:
-            csvwriter = csv.writer(csvfile, delimiter=';')
-
-            with open(output_path2file_temp) as f:
-                headers_object = ijson.items(f, "data.fields")
-                for headers in headers_object:
-                    csv_headers = [header["header"] for header in headers]
-                    csvwriter.writerow(csv_headers)
-              
-            with open(output_path2file_temp) as f:
-                rows_object = ijson.items(f, "data.rows")
-                for rows in rows_object:
-                    for row in rows:
-                        csvwriter.writerow(row)
-
-        # removing temp file
-        if os.path.exists(output_path2file_temp):
-            self.__log(f"Deleting temp_file={output_path2file_temp}")
-            os.remove(output_path2file_temp)
-
-        self.__log(f"Output csv path2file={output_path2file}")
-        return output_path2file
+                l_path2file.append(d_output_path2file["path2file"])
 
+        return l_path2file
 
     def download_edw(
             self,
@@ -301,14 +380,15 @@ class Conn:
         def save_get_edw_token():
             token_expiry_epoch = int(time.time()) + 3600 * 11.5
             edw_token_url = f"{self.__base_url}/ea/v2/er/account/get_dw_session_token.json"
+            payload = { 'ip' : ip }
             edw_token_json = requests.get(
                 url=edw_token_url,
                 headers=self.__http_headers,
-                params={'ip' : ip}
+                params=payload
             ).json()
             
             if self.__has_api_error(edw_token_json):
-                return 0
+                raise SystemError(f"Error for url={edw_token_url}?{urllib.parse.urlencode(payload)}")
 
             edw_token = edw_token_json["data"]["rows"][0][0]
             token_json = { "token" : edw_token, "expiry" : token_expiry_epoch, "ip" : ip }
@@ -390,7 +470,7 @@ class Conn:
             ).json()
 
             if self.__has_api_error(search_json):
-                return 0
+                raise SystemError(f"Error for url={search_url}")
 
             jobrun_id = search_json['data'][0]
 
@@ -410,7 +490,7 @@ class Conn:
             ).json()
 
             if self.__has_api_error(status_json):
-                return 0
+                raise SystemError(f"Error for url={status_url}")
 
             if status_json["status"] == "Done":
                 ready = True
@@ -488,8 +568,7 @@ class Conn:
         Returns
         -------
         dict
-            Success : A dict as { "view_id" : "view_name", ...}
-            Failure : { "error" : 1 }
+            A dict as { "view_id" : "view_name", ...}
         """
 
         view_url = f"{self.__base_url}/ea/v2/ea/{website_name}/db/view/get_all_name.json"
@@ -499,7 +578,7 @@ class Conn:
         ).json()
 
         if self.__has_api_error(view_json):
-            return  { "error" : 1 }
+            raise SystemError(f"Error for url={view_url}")
 
         view_id_idx = view_json["data"]["fields"].index({"name" : "view_id"})
         view_name_idx = view_json["data"]["fields"].index({"name" : "view_name"})
